import numpy
from pysparse import spmatrix
import sparsetools
import pycfs
import pyma27
import norms
import math
from nlpy_timing import cputime
#import profile
#import pstats

#ProblemList = [ '1138bus.mtx', 'bcsstk08.mtx', 'bcsstk09.mtx', 'bcsstk10.mtx', 'bcsstk11.mtx', 'bcsstk18.mtx', 'bcsstk19.mtx' ]
ProblemList = [ 'Data/bcsstk08.mtx' ]

def test_pycfs():
    for problem in ProblemList:

        A = spmatrix.ll_mat_from_mtx( problem )
        (m, n) = A.shape

        print '=' * 70
        print ' %s' % problem
        print ' matrix size : %d x %d' % (m, n)
        print ' nnz(A) = ', A.nnz

        # Right-hand side is Ae, as in Icfs,
        # thus, the exact solution should be x = e.
        e = numpy.ones( n, 'd' )
        b = numpy.ones( m, 'd' )
        A.matvec( e, b )

        # Solve with Pyma27
        t = cputime()
        P = pyma27.PyMa27Context( A )
        t_factor27 = cputime() - t
        t = cputime()
        P.solve( b )
        t_solve27 = cputime() - t
        x27 = P.x
        r27 = P.residual
        nr27 = norms.norm2( r27 )
        err27 = norms.norm2( x27 - e )/math.sqrt( n )

        max_dist = 0.
        
        # Call icfs and pcg
        for pval in [0, 5, 10]:
            t = cputime()
            Q = pycfs.PycfsContext( A, mem = pval )
            t_fact = cputime() - t
            t = cputime()
            Q.solve( b )
            t_solve = cputime() - t
            x = Q.x
            err = norms.norm2( x - e )/math.sqrt( n )

            # Compute distance between two solutions
            max_dist = max( max_dist, norms.norm_infty( x - x27 ) )

            print '   Pycfs'
            print '     memory p = ', pval
            print '     info = ', Q.info
            if Q.info == 2:
                print '     negc = ', Q.nc
            print '     iter = ', Q.iter
            print '     rres = ', Q.relres
            print '     rel. error: ', err
            print '     Fact. time: ', t_fact
            print '     Solve time: ', t_solve
            print '     total time: ', t_fact + t_solve

        print '   MA27'
        print '     residual  : ', nr27
        print '     rel. error: ', err27
        print '     Fact. time: ', t_factor27
        print '     Solve time: ', t_solve27
        print ''
        print ' Max distance between solutions: ', max_dist

# Do it!
test_pycfs()

#profile.run( 'test_pycfs()', 'prof.results' )
#p = pstats.Stats( 'prof.results' )
#p.strip_dirs().sort_stats(-1).print_stats()
#p.sort_stats( 'name' )
#p.print_stats()
