# Simple example:
#    steepest descent method
#    with Armijo backtracking
#
# See NLPy/steepest.py for a more general class

from numpy import array, dot
from norms import norm_infty
from linesearch import ArmijoLineSearch
from rosenbrock import RosenbrockModel
    
#def rosenbrock(x):
#    return 10.0 * (x[1]-x[0]**2)**2 + (1-x[0])**2

def rosenbrockxy(x,y):
    return RosenbrockModel.obj( (x,y) )

#def rosengrad(x):
#    return array( [ -40.0 * (x[1] - x[0]**2) * x[0] - 2.0 * (1-x[0]),
#                    20.0 * (x[1] - x[0]**2) ], 'd' )

ALS = ArmijoLineSearch( tfactor = 0.2 )
x = array( [-0.5, 1.0], 'd' )
f = RosenbrockModel.obj(x)
grad = RosenbrockModel.grad(x)
d = -grad
slope = dot(grad, d)
t = 0.0
tlist = []
xlist = [x[0]]
ylist = [x[1]]
iter = 0
format = '%-3d  %-12g  %-12g  %-12g  %-12g  %-6g  %-12g'
print format % (iter, f, norm_infty(grad), x[0], x[1], t, slope)
while norm_infty(grad) > 1.0e-5:

    iter += 1

    # Perform linesearch
    t = ALS.search(RosenbrockModel.obj, x, d, slope, f=f)
    tlist.append(t)

    # Move ahead
    x += t * d
    xlist.append(x[0])
    ylist.append(x[1])
    f = RosenbrockModel.obj(x)
    grad = RosenbrockModel.grad(x)
    d = -grad
    slope = dot(grad, d)
    print format % (iter, f, norm_infty(grad), x[0], x[1], t, slope)

try:
    from pylab import *
except:
    import sys
    sys.stderr.write( 'If you had Matplotlib, you would be looking\n' )
    sys.stderr.write( 'at a contour plot right now...\n' )
    sys.exit(0)

xx = arange( -1.0, 1.1, 0.01 )
yy = arange( 0.0, 1.2, 0.01 )
XX, YY = meshgrid( xx, yy )
ZZ = rosenbrockxy( XX, YY )
plot( xlist, ylist, 'r-', lw = 1.5 )
plot( [xlist[0]], [ylist[0]], 'go', [xlist[-1]], [ylist[-1]], 'go' )
contour( XX, YY, ZZ, linewidths = 1.5, alpha = 0.75, origin = 'lower' )
title( 'Steepest descent with Armijo linesearch on the Rosenbrock function' )
show()
#savefig( 'steep-rosenbrock.png' )
