import amplpy
import sys

# Helper function to display a linear function in algebraic
# format given its gradient in sparse vector form.
def display_linear( row ):
    keys = row.keys()
    keys.sort()
    s = ''
    for k in keys:
        val = row[k]
        if val < 0:
            s += ' - ' + str( abs( val ) )
        else:
            if s == '':
                s += str( val )
            else:
                s += ' + ' + str( val )
        s += ' x%-d' % k
    return s

def usage():
    print 'Use %-s lp' % sys.argv[0]
    print '  where lp is the name of an Ampl model'
    print '  representing a linear programming problem.'

if len( sys.argv ) < 2:
    usage()
    sys.exit( 1 )

# Load model
probname = sys.argv[1]
lp = amplpy.AmplModel( probname )

islp = lp.islp()
print ' Problem %-s is an LP : %-s' % (probname, islp)

if not islp:
    print ' This demo concerns only linear programs... exiting.'
    sys.exit( 2 )

c = lp.cost()
print '  Objective:\n   %-s' % display_linear( c )

if lp.nbounds:
    print '  Bound constraints:'
    for i in lp.lowerB:
        print '   %-10g <= x[%-5d]' % (lp.Lvar[i], i)
    for i in lp.upperB:
        print '   %-10s    x[%-5d] <= %-10g' % ('', i, lp.Uvar[i])
    for i in lp.rangeB:
        print '   %-10g <= x[%-5d] <= %-10g' % (lp.Lvar[i], i, lp.Uvar[i])
else:
    print '  No bound constraints were given'

if lp.m:
    print '  Linear constraints'
    for i in lp.lowerC:
        s = display_linear( lp.irow( i ) )
        print '   %-10g <= %-s' % (lp.Lcon[i], s)
    for i in lp.upperC:
        s = display_linear( lp.irow( i ) )
        print '   %-10s    %-s <= %-10g' % ('', s, lp.Ucon[i])
    for i in lp.rangeC:
        s = display_linear( lp.irow( i ) )
        print '   %-10g <= %-s <= %-10g' % (lp.Lcon[i], s, lp.Ucon[i])
    print
    print ' A = ', lp.A()
else:
    print '  No linear constraints were given'

# Terminate
print ' End'
lp.close()
