"""
LSQR demo.
$Id$
"""

from numpy import arange, dot, concatenate, zeros
from numpy.linalg import norm
from lsqr import lsqr


def aprod(mode, m, n, z):
    """
    if mode = 1, lsqraprod computes y = A*z
    if mode = 2, lsqraprod computes y = A'*z
    for some matrix  A.
    
    This is a simple example for testing  LSQR.
    It uses the leading m*n submatrix from
    A = [ 1
          1 2
            2 3
              3 4
                ...
                  n ]
    suitably padded by zeros.
    """
    if mode == 1:
        d  = arange(1,n+1)
        y1 = concatenate((d*z, [0])) + concatenate(([0],d*z))
        if m <= n+1:
            w = y1[:m]
        else:
            w = concatenate((y1, zeros(m-n-1)))
    else:
        d  = arange(1,m+1)
        y1 = d*z + concatenate((d[:m-1]*z[1:m], [0]))
        if m >= n:
            w = y1[:n]
        else:
            w = concatenate((y1, zeros(n-m)))

    return w

def lsqrtest(m,n,damp):
    """
    If m = n and damp = 0, this sets up a system Ax = b
    and calls lsqr to solve it.  Otherwise, the usual
    least-squares or damped least-squares problem is solved.
    """
    xtrue  = arange(n,0,-1)
    b = aprod( 1, m, n, xtrue );

    atol   = 1.0e-6
    btol   = 1.0e-6
    conlim = 1.0e+10
    itnlim = 10*n
    show   = True

    ( x, istop, itn, r1norm, r2norm, anorm, acond, arnorm, xnorm, var ) = \
        lsqr( m, n, aprod, b, damp, atol, btol, conlim, itnlim, show )

    j1 = min(n,5)
    j2 = max(n-4,1)
    print 'First elements of x: ' , x[:j1]
    print 'Last  elements of x: ' , x[j2:n]
    r = b - aprod(1,m,n,x)

    return False

if __name__ == "__main__":

    from nlpy_timing import cputime
    (m, n) = (1000,200)
    t = cputime()
    lsqrtest(m,n,0)
    print 'Total runtime (sec)', cputime() - t
