import pyma27, norms
from pysparse import umfpack, spmatrix
import numpy
from sparsetools import Hilbert
from demo_ma27 import Ma27SpecSheet

def testMa27Fact( A, b, name ):
    """
    Solve the system Ax = b in two ways with the pyma27 module:
    1.  Let ma27 factor and solve.
    2.  Use ma27 to return the factors  L and inv(B),
        then recover the solution via
	x = L'\( inv(B) * \ (L\b) ).
    Print some statistics.
    """
    n = A.shape[0]

    # Factor the matrix.
    LBL = pyma27.PyMa27Context(A)
    LBL.fetch_lb()

    # Compute the solution using Ma27.
    LBL.solve(b)
    
    # Compute the solution using the factors.
    x  = numpy.zeros(n, 'd')
    y  = numpy.zeros(n, 'd')
    z  = numpy.zeros(n, 'd')
    LU = umfpack.factorize( LBL.L )
    LU.solve( b, z, 'UMFPACK_A' )
    LBL.B.matvec( z, y )
    LU.solve( y, x, 'UMFPACK_At' )
    
    # Print the matrix and its factors.
    print "=== %s ===" % name
    if n <= 5:
        print A
        print "L =", LBL.L
        perm = LBL.fetch_perm()
        LP = spmatrix.ll_mat( n, n, LBL.L.nnz )
        k = LBL.L.keys()
        for elem in k:
            i = elem[0]
            j = elem[1]
            print i, j, perm[i]-1, perm[j]-1
            LP[ perm[i]-1, perm[j]-1 ] = LBL.L[i-1,j-1]
        print LP
        print "inv(B) =", LBL.B

    # Print the "direct" and "indirect" solution.
    err = norms.norm_infty( LBL.x - x )
    print "Error between Ma27's solution, and solution computed via LBL' factors: %10.2e" % err

if __name__ == "__main__":
    # Test the Ma27 specsheet matrix:
    (A, b) = Ma27SpecSheet()
    testMa27Fact( A, b, 'Ma27SpecSheet')

    # Test a Hilbert matrix:
    n = 5
    A = Hilbert(n)
    e = numpy.ones( n, 'd' )
    b = numpy.zeros( n, 'd' )
    A.matvec( e, b )
    testMa27Fact( A, b, 'Hilbert (%i)' % n )

    import sys
    import os
    matrices = sys.argv[1:]

    from pysparse import spmatrix

    # Process matrices given on the command line
    for matrix in matrices:
        M = spmatrix.ll_mat_from_mtx( matrix )
        (m,n) = M.shape
        if m != n: break
        e = numpy.ones( n, 'd' )
        rhs = numpy.zeros( n, 'd' )
        M.matvec( e, rhs )
        testMa27Fact( M, rhs, matrix )
