# Illustrate usage of module nlp
# Implement a very simple Newton method on a quadratic function

import nlp

# We can define an objective to be minimized
def my_obj( x ): return float(x)**2

# Or a nonlinear equation to be solved
def my_obj_eqn( x ): return float(x)**2 - 3

# For optimization, we can use this measure of optimality,
def my_resid( x, z, **kwargs ): return (abs( my_grad( x ) ), 0.0, 0.0)

# For a nonlinear equation, we could use this one
def my_resid_eqn( x, z, **kwargs ): return (abs( my_obj_eqn( x ) ), 0.0, 0.0)

# In the above two cases, the derivatives are the same
def my_grad( x ): return 2.0*x
def my_hess( x, z ): return 2.0
def my_hprod( x, z, p ): return 2.0*p

# Create new problem instance
prob = nlp.NLPModel( n = 1, m = 0, name = 'blah' )

# Customize problem methods
prob.obj = my_obj_eqn     # Or my_obj_eqn
prob.grad = my_grad
prob.hess = my_hess
prob.hprod = my_hprod

# Customize optimality test
# The following function is called by the
# 'AtOptimality' method to test for optimality
prob.OptimalityResiduals = my_resid_eqn    # Or my_resid

def Newton( model, x ):
    print '%-5s  %-12s  %-12s  %-12s' % ( 'Iter', 'x', 'f(x)', '|g(x)|' )
    print '-' * 47
    i = 0
    while not model.AtOptimality( x, None ):
        print '%-5d  %-12.10g  %-12g  %-12g' % ( i, x, model.obj( x ), model.grad( x ) )
        x = x - model.obj( x )/model.grad( x )
        i += 1
    print '%-5d  %-12.10g  %-12g  %-12g' % ( i, x, model.obj( x ), model.grad( x ) )
    print '-' * 47
    return x

x0 = 2.0
x = Newton( prob, x0 )

