from pysparse import spmatrix
import sparsetools
import pycfs
import sys

def usage():
    progname = sys.argv[0]
    sys.stderr.write( 'Usage\n' );
    sys.stderr.write( '  %-s problem [problem ...]\n' % progname )
    sys.stderr.write( '  where each problem is the file name of a matrix\n' )
    sys.stderr.write( '  in MatrixMarket sparse format (*.mtx).\n' )

def pycfs_fact( ProblemList, bigp = 500 ):
    import os

    if len( ProblemList ) == 0:
        usage()
        sys.exit( 1 )
    
    header = '%-12s  %-6s  %-6s  %-7s  %-8s  %-8s  %-8s  %-10s  %-4s' % ('Name', 'n', 'nnz(A)', 'nnz(L)', 'densityA', 'densityL', 'shift', 'mem', 'p*' )
    lhead = len( header )
    print header
    print '-' * lhead

    for problem in ProblemList:

        A = spmatrix.ll_mat_from_mtx( problem )
        (m, n) = A.shape
        if m != n: break
        #sparsetools.spymatll( A )

        prob = os.path.basename( problem )
        if prob[-4:] == '.mtx': prob = prob[:-4]

        P = pycfs.PycfsContext( A, mem = bigp )
        L = P.fetch()  # Retrieve Cholesky factor
        nnzA = A.nnz
        nnzL = L.nnz
        densityA = 100.0 * nnzA/(n*(n+1)/2.0)
        densityL = 100.0 * nnzL/(n*(n+1)/2.0)
        shift = P.shift
        memory_available = n*bigp
        p_nnz = nnzL/n
        #sparsetools.spymatll( L )

        print '%-12s  %-6d  %-6d  %-7d  %-8g  %-8g  %-8g  %-10d  %-4d' % (prob, n, nnzA, nnzL, densityA, densityL, shift, memory_available, p_nnz)

    print '-' * lhead
    return None

if __name__ == '__main__':

    pycfs_fact( sys.argv[1:] )    
