# An instance of an NLP implementing
# the Rosenbrock "banana" function
# D. Orban, Oct. 2005.

import numpy
from nlp import NLPModel
from norms import norm_infty

def rosenbrock_obj(x):
    return 100.0 * (x[1]-x[0]**2)**2 + (1-x[0])**2

def rosenbrock_grad(x):
    return numpy.array( [-400.0 * (x[1] - x[0]**2) * x[0] - 2.0 * (1-x[0]),
                         200.0 * (x[1] - x[0]**2)], 'd' )

def rosenbrock_hess(x):
    return numpy.array( [ [-400.0 * x[1] + 1200.0 * x[0]**2 + 2, -400.0 * x[0]],
                          [-400.0 * x[0], 200.0] ], 'd' )

def rosenbrock_hprod(x,z,d):
    return numpy.array( [ (-400.0*x[1]+1200.0*x[0]**2+2)*d[0] -400.0*x[0]*d[1],
                           -400.0*x[0]*d[0] + 200.0*d[1] ] )


# Define rosenbrock object as unconstrained NLPModel
# and assign default starting point.
RosenbrockModel = NLPModel( n = 2,
                            m = 0,
                            name = 'rosenbrock',
                            x0 = numpy.array( [-1.5, 1.0], 'd' ) )
    
# Assign objective function, gradient and Hessian
RosenbrockModel.obj  = rosenbrock_obj
RosenbrockModel.grad = rosenbrock_grad
RosenbrockModel.hess = rosenbrock_hess
RosenbrockModel.hprod = rosenbrock_hprod

# Assign default measure of optimality
def optimality_residuals( x, z, **kwargs ):
    # If gradient at x is not given, evaluate it
    # This can save the trouble of extra gradient
    # evaluations
    if 'gradient' in kwargs.keys():
        g = kwargs['gradient']
    else:
        g = rosenbrock.grad( x )
    return ( norm_infty( g ), 0.0, 0.0 )

RosenbrockModel.OptimalityResiduals = optimality_residuals
